@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the tools applications of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "mainwindow.h"
 #include "changeproperties.h"
@@ -47,8 +22,7 @@
 #include <QtAxContainer/QAxScriptManager>
 #include <QtAxContainer/QAxWidget>
 #include <QtAxContainer/private/qaxbase_p.h>
-#include <memory>
-#include <sddl.h>
+#include "sandboxing.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -115,70 +89,28 @@ QList<QAxWidget *> MainWindow::axWidgets() const
     return result;
 }
 
-
-/** RAII class for temporarily impersonating low-integrity level for the current thread.
-    Intended to be used together with CLSCTX_ENABLE_CLOAKING when creating COM objects.
-    Based on "Designing Applications to Run at a Low Integrity Level" https://msdn.microsoft.com/en-us/library/bb625960.aspx */
-struct LowIntegrity {
-    LowIntegrity()
-    {
-        HANDLE cur_token = nullptr;
-        if (!OpenProcessToken(GetCurrentProcess(), TOKEN_DUPLICATE | TOKEN_ADJUST_DEFAULT | TOKEN_QUERY | TOKEN_ASSIGN_PRIMARY, &cur_token))
-            abort();
-
-        if (!DuplicateTokenEx(cur_token, 0, nullptr, SecurityImpersonation, TokenPrimary, &m_token))
-            abort();
-
-        CloseHandle(cur_token);
-
-        PSID li_sid = nullptr;
-        if (!ConvertStringSidToSid(L"S-1-16-4096", &li_sid)) // low integrity SID
-            abort();
-
-        // reduce process integrity level
-        TOKEN_MANDATORY_LABEL TIL = {};
-        TIL.Label.Attributes = SE_GROUP_INTEGRITY;
-        TIL.Label.Sid = li_sid;
-        if (!SetTokenInformation(m_token, TokenIntegrityLevel, &TIL, sizeof(TOKEN_MANDATORY_LABEL) + GetLengthSid(li_sid)))
-            abort();
-
-        if (!ImpersonateLoggedOnUser(m_token)) // change current thread integrity
-            abort();
-
-        LocalFree(li_sid);
-        li_sid = nullptr;
-    }
-
-    ~LowIntegrity()
-    {
-        if (!RevertToSelf())
-            abort();
-
-        CloseHandle(m_token);
-        m_token = nullptr;
-    }
-
-private:
-    HANDLE m_token = nullptr;
-};
-
 bool MainWindow::addControlFromClsid(const QString &clsid, QAxSelect::SandboxingLevel sandboxing)
 {
     QAxWidget *container = new QAxWidget;
 
     bool result = false;
     {
-        std::unique_ptr<LowIntegrity> low_integrity;
+        // RAII object for impersonating sandboxing on current thread
+        std::unique_ptr<Sandboxing> sandbox_impl;
 
-        if (sandboxing == QAxSelect::SandboxingProcess) {
+        switch (sandboxing) {
+        case QAxSelect::SandboxingNone:
+            break; // sandboxing disabled
+        case QAxSelect::SandboxingProcess:
             // require out-of-process
             container->setClassContext(CLSCTX_LOCAL_SERVER);
-        } else if (sandboxing == QAxSelect::SandboxingLowIntegrity) {
-            // impersonate "low integrity"
-            low_integrity.reset(new LowIntegrity);
-            // require out-of-process and
-            // propagate integrity level when calling setControl
+            break;
+        default:
+            // impersonate desired sandboxing
+            sandbox_impl = Sandboxing::Create(sandboxing, clsid);
+            // require out-of-process and activate impersonation
             container->setClassContext(CLSCTX_LOCAL_SERVER | CLSCTX_ENABLE_CLOAKING);
+            break;
         }
 
         result = container->setControl(clsid);
